﻿namespace Phoenix.Main.Frame
{
    using System;
    using System.Diagnostics;
    using System.Drawing;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Log.Writer;
    using Phoenix.Main.Configuration;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.View.Events;
    using Phoenix.View.Sessions;
    using Phoenix.Translation;
    using Phoenix.View;
    using Phoenix.View.Helpers;
    using Phoenix.View.Management;

    using WeifenLuo.WinFormsUI.Docking;

    /// <summary>
    /// Represent the side sideMenu to administrate the whole application.
    /// </summary>
    public partial class SideMenu : TranslatedForm
    {
        #region Fields

        private DockPanel dockPanel;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SideMenu"/> class.
        /// </summary>
        public SideMenu(DockPanel dockPanel)
        {
            InitializeComponent();
            this.dockPanel = dockPanel;
            this.Logger = LogManager.GetLogger(typeof(SideMenu));
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when this DockContent is hidden.
        /// </summary>
        public event EventHandler Hidden;

        /// <summary>
        /// Occurs when the user clicks on the "search" button of the openedSessions manager.
        /// </summary>
        public event EventHandler SearchPatientClicked;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the list of the managers of the opened session.
        /// </summary>
        /// <value>A list of SessionGui.</value>
        public SessionGuiCollection SessionsCollection
        {
            get;
            private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Adds a new open sessions in the menu.
        /// </summary>
        /// <param name="session">The session.</param>
        /// <returns><c>True</c> if the session is open. Otherwise,
        /// if a session is already opened, returns <c>False</c></returns>
        public bool Add(Session session)
        {
            if (this.SessionsCollection.Contains(session))
            {
                string msg = "A session is already open for '{0}'";
                MessageBoxes.Warning(msg.Translate(session.Patient.ToString()));
                return false;
            }
            else
            {
                session.FamilyManagerModified += session_FamilyManagerModified;
                session.PatientSelected += session_PatientSelected;
                InsertInTreeView(session);
                this.SessionsCollection.Add(session);
                return true;
            }
        }

        /// <summary>
        /// Refreshes all the opened managers.
        /// </summary>
        public void RefreshManagers()
        {
            foreach (var session in this.SessionsCollection)
            {
                Control control = session.PatientManager as Control;
                if (control != null) control.Refresh();

                control = session.FamilyManager as Control;
                if (control != null) control.Refresh();

                control = session.BmiManager as Control;
                if (control != null) control.Refresh();
            }
        }

        /// <summary>
        /// Refreshes the session to remove all the closed sessions.
        /// </summary>
        public void RefreshSessions()
        {
            SessionGuiCollection sessionsBuffer = new SessionGuiCollection();
            foreach (Session item in this.SessionsCollection)
            {
                if (!item.IsClosed)
                    sessionsBuffer.Add(item);
            }
            this.SessionsCollection = sessionsBuffer;

            this.tv_Sessions.SuspendLayout();
            this.tv_Sessions.Nodes.Clear();
            this.InsertInTreeView(sessionsBuffer);
            this.tv_Sessions.ResumeLayout();
        }

        /// <summary>
        /// Removes the specified openedSessions from the menu and close this openedSessions.
        /// </summary>
        /// <param name="session">The session to remove.</param>
        public void Remove(Session session)
        {
            if (!this.SessionsCollection.Contains(session))
            {
                string msg = "No session exists for '{0}'";
                MessageBoxes.Warning(msg.Translate(session.Patient));
                base.Logger.Warning(msg, session.Patient);
                return;
            }

            base.Logger.Debug("Remove session '{0}'", session.Patient.ID);
            this.UnsubscribeEvents(session);
            this.SessionsCollection.Remove(session);

            foreach (TreeNode node in this.tv_Sessions.Nodes)
            {
                Session otherSession = node.Tag as Session;
                if (otherSession == session)
                {
                    Logger.Debug("Remove node with session '{0}'", ((Session)node.Tag).Patient.ID);
                    tv_Sessions.Nodes.Remove(node);
                    return;
                }
            }
        }

        private void btn_SearchSession_Click(object sender, EventArgs e)
        {
            OnSessionClicked();
        }

        private void InsertInTreeView(Session session)
        {
            string name = string.Format("{0} {1}", session.Patient.FirstName, session.Patient.LastName);
            TreeNode item = new TreeNode(name);
            item.ImageKey = "Root";
            item.Tag = session;

            TreeNode subitem = new TreeNode("Patient information".Translate());
            subitem.ImageKey = "Patient";
            item.Nodes.Add(subitem);

            subitem = new TreeNode("Family".Translate());
            subitem.ImageKey = "Family";
            item.Nodes.Add(subitem);

            subitem = new TreeNode("Medical cards".Translate());
            subitem.ImageKey = "MedicalCard";
            item.Nodes.Add(subitem);

            subitem = new TreeNode("Bmi history".Translate());
            subitem.ImageKey = "Bmi";
            item.Nodes.Add(subitem);

            subitem = new TreeNode("Pictures".Translate());
            subitem.ImageKey = "Picture";
            item.Nodes.Add(subitem);

            subitem = new TreeNode("Prescriptions".Translate());
            subitem.ImageKey = "Prescription";
            item.Nodes.Add(subitem);

            subitem = new TreeNode("Pathologies".Translate());
            subitem.ImageKey = "Pathology";
            item.Nodes.Add(subitem);

            subitem = new TreeNode("Meetings".Translate());
            subitem.ImageKey = "Meeting";

            tv_Sessions.Nodes.Add(item);
        }

        private void InsertInTreeView(SessionGuiCollection sessions)
        {
            foreach (Session item in sessions)
            {
                this.InsertInTreeView(item);
            }
        }

        private void OnSessionClicked()
        {
            this.SearchPatientClicked.Raise(this);
        }

        private void session_FamilyManagerModified(object sender, EventArgs e)
        {
            foreach (var session in this.SessionsCollection)
            {
                string msg = string.Format("Family update: refresh session {0}.", session.Patient.ID);
                Logger.Info(msg);
                session.RefreshFamilyManager();
            }
        }

        private void session_PatientSelected(object sender, EventArgs<Patient> e)
        {
            Session session = new Session(e.Value);
            this.Add(session);
        }

        private void SideMenu_DockStateChanged(object sender, EventArgs e)
        {
            this.Logger.Debug(string.Format("DockState changed to: {0}", this.DockState));
            if (this.DockState == DockState.Hidden)
            {
                this.Hidden.Raise(this);
            }
        }

        private void SideMenu_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Logger.Debug("Side Menu raised FormClosing...");
            base.Hide();
            this.Hidden.Raise(this);
            e.Cancel = true;
        }

        private void SideMenu_Load(object sender, EventArgs e)
        {
            this.SessionsCollection = new SessionGuiCollection();

            this.Text = "Side menu".Translate();
            this.btn_SearchSession.Text = "Search...".Translate();

            this.nSessions.Text = "Sessions".Translate();
            this.nMeetings.Text = "Meetings".Translate();
            this.nStatistics.Text = "Statistics".Translate();
            this.nAdministration.Text = "Administration".Translate();
        }

        private void tsmi_CloseSession_Click(object sender, EventArgs e)
        {
            Session session = tv_Sessions.SelectedNode.Tag as Session;
            Logger.Info("Removing process started...");
            if (session != null)
            {
                session.Save();
                this.Remove(session);
            }
            else
            {
                MessageBoxes.Warning("No patient selected!");
            }
        }

        private void tsmi_Collapse_Click(object sender, EventArgs e)
        {
            this.tv_Sessions.CollapseAll();
        }

        private void tsmi_DeletePatient_Click(object sender, EventArgs e)
        {
            Session session = tv_Sessions.SelectedNode.Tag as Session;
            Contract.Assert((session != null), "The selected session shouldn't be null");

            Patient patient = session.Patient;
            PatientDeletionBox box = new PatientDeletionBox(patient);
            DialogResult dr = box.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.Remove(session);
            this.RefreshSessions();
            this.RefreshManagers();
        }

        private void tsmi_Expand_Click(object sender, EventArgs e)
        {
            this.tv_Sessions.ExpandAll();
        }

        private void tv_Sessions_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            e.Node.SelectedImageKey = e.Node.ImageKey;
        }

        private void tv_Sessions_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                tv_Sessions.SelectedNode = tv_Sessions.GetNodeAt(e.X, e.Y);
            }
        }

        private void tv_Sessions_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                Point clickedPoint = new Point(e.X, e.Y);
                TreeNode clickedNode = tv_Sessions.GetNodeAt(clickedPoint);
                if (clickedNode == null || !(clickedNode.Tag is Session))
                    return;
                else
                {
                    // Convert from Tree coordinates to Screen coordinates
                    Point screenPoint = tv_Sessions.PointToScreen(clickedPoint);
                    // Convert from Screen coordinates to Form coordinates
                    Point formPoint = this.PointToClient(screenPoint);
                    // Show context menu
                    cms_ContextSession.Show(this, formPoint);
                }
            }
        }

        private void tv_Sessions_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            #region MedicalCard
            if (e.Node.ImageKey == "MedicalCard")
            { }
            #endregion
            #region Patient
            else if (e.Node.ImageKey == "Patient")
            {
                AppearanceConfiguration appearanceConfig = new AppearanceConfiguration();
                GeneralConfiguration generalConfig = new GeneralConfiguration();

                Session session = this.SessionsCollection[(Session)e.Node.Parent.Tag];

                PatientManagerBox manager = session.PatientManager as PatientManagerBox;

                if (manager == null)
                {
                    PatientManagerBox.Configuration configuration = new PatientManagerBox.Configuration()
                    {
                        CriterionPanelHeight = appearanceConfig.PatientCriterionPanelHeight,
                        SearchBoxView = appearanceConfig.SearchBoxView,
                        FrameAction = FrameAction.Update,
                        MobileNumberDisplayPattern = generalConfig.MobileNumberDisplayPattern,
                        PhoneNumberDisplayPattern = generalConfig.PhoneNumberDisplayPattern
                    };
                    PatientManagerBox managerBox = new PatientManagerBox(session.Patient, configuration);

                    managerBox.IsTabMdi = true;

                    session.PatientManager = managerBox;
                    manager = session.PatientManager as PatientManagerBox;
                }
                manager.Show(this.dockPanel);
            }
            #endregion
            #region Bmi
            else if (e.Node.ImageKey == "Bmi")
            {
                GeneralConfiguration config = new GeneralConfiguration();
                Session session = this.SessionsCollection[(Session)e.Node.Parent.Tag];
                BmiManagerBox manager = session.BmiManager as BmiManagerBox;

                if (manager == null)
                {
                    session.BmiManager = new BmiManagerBox(session.Patient, config.DateFormat);
                    manager = session.BmiManager as BmiManagerBox;
                }
                manager.Show(this.dockPanel);
            }
            #endregion
            #region Family
            else if (e.Node.ImageKey == "Family")
            {
                GeneralConfiguration config = new GeneralConfiguration();
                Session session = this.SessionsCollection[(Session)e.Node.Parent.Tag];
                FamilyManagerBox manager = session.FamilyManager as FamilyManagerBox;

                if (manager == null)
                {
                    session.FamilyManager = new FamilyManagerBox(session.Patient);
                    manager = session.FamilyManager as FamilyManagerBox;
                }
                manager.Show(this.dockPanel);
            }
            #endregion
        }

        private void UnsubscribeEvents(Session session)
        {
            session.PatientSelected -= session_PatientSelected;
            session.FamilyManagerModified -= session_FamilyManagerModified;
        }

        #endregion Methods
    }
}