﻿namespace Phoenix.Model.Bll.Domain
{
    using System;

    using Phoenix.Contracts;

    public class BMI : Poco
    {
        #region Enumerations

        /// <summary>
        /// Represent the Bmi status corresponding to the index
        /// </summary>
        public enum BmiStatus
        {
            /// <summary>
            /// Obese Class III
            /// </summary>
            ObeseClass3,
            /// <summary>
            /// Obese Class II
            /// </summary>
            ObeseClass2,
            /// <summary>
            /// Obese Class I
            /// </summary>
            ObeseClass1,
            /// <summary>
            /// Overweight
            /// </summary>
            Overweight,
            /// <summary>
            /// Normal
            /// </summary>
            Normal,
            /// <summary>
            /// Underweight
            /// </summary>
            Underweight,
            /// <summary>
            /// Severe Underweight
            /// </summary>
            SevereUnderweight,
            /// <summary>
            /// The BMI index is unspecified because one of the values of the BMI is not correct.
            /// </summary>
            Unspecified,
        }

        #endregion Enumerations

        #region Properties

        /// <summary>
        /// Gets or sets the date of the bmi.
        /// </summary>
        /// <value>The date.</value>
        public virtual DateTime Date
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the height.
        /// </summary>
        /// <value>The height of the patient.</value>
        public virtual int Height
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the calculated BMI index.
        /// </summary>
        /// <value>The BMI index.</value>
        public virtual double Index
        {
            get
            {
                if (this.Height <= 0 || this.Weight <= 0)
                    return 0;
                else
                    return Math.Round((this.Weight / Math.Pow(((float)this.Height / 100), 2)), 1);
            }
        }

        /// <summary>
        /// Gets the status of the BMI index.
        /// </summary>
        /// <value>The status.</value>
        public virtual BmiStatus Status
        {
            get
            {
                if (this.Index == 0)
                {
                    return BmiStatus.Unspecified;
                }
                else if (this.Index > 40)
                {
                    return BmiStatus.ObeseClass3;
                }
                else if (this.Index >= 35 && this.Index <= 40)
                {
                    return BmiStatus.ObeseClass2;
                }
                if (this.Index >= 30.1 && this.Index <= 34.9)
                {
                    return BmiStatus.ObeseClass1;
                }
                else if (this.Index >= 25 && this.Index <= 30)
                {
                    return BmiStatus.Overweight;
                }
                else if (this.Index >= 18.5 && this.Index <= 24.9)
                {
                    return BmiStatus.Normal;
                }
                else if (this.Index >= 16.5 && this.Index <= 18.4)
                {
                    return BmiStatus.Underweight;
                }
                else if (this.Index < 16.5)
                {
                    return BmiStatus.SevereUnderweight;
                }
                return BmiStatus.Unspecified;
            }
        }

        /// <summary>
        /// Gets the status into a string. The status is not translated and
        /// the default language is English!
        /// </summary>
        /// <value>The text status.</value>
        public virtual string TextStatus
        {
            get
            {
                switch (this.Status)
                {
                    case BmiStatus.ObeseClass3:
                        return "Obese Class III";
                    case BmiStatus.ObeseClass2:
                        return "Obese Class II";
                    case BmiStatus.ObeseClass1:
                        return "Obese Class I";
                    case BmiStatus.Overweight:
                        return "Overweight";
                    case BmiStatus.Normal:
                        return "Normal";
                    case BmiStatus.Underweight:
                        return "Underweight";
                    case BmiStatus.SevereUnderweight:
                        return "Severely underweight";
                    case BmiStatus.Unspecified:
                        return "Unspecified";
                    default:
                        return (string)Assert.Fails("The status '{0}' is not supported!", this.Status);
                }
            }
        }

        /// <summary>
        /// Gets or sets the weight of the patient.
        /// </summary>
        /// <value>The weight.</value>
        public virtual double Weight
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return string.Format("{0} - {1} cm - {2} kg", this.Date.ToString("dd/MM/yyyy"), this.Height, this.Weight);
        }

        #endregion Methods
    }
}