﻿namespace Phoenix.Model.Management
{
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Presenter;

    /// <summary>
    /// Provides basic feature of a manager. A manager is a sub feature of a session such as
    /// Picture manager or Meeting manager.
    /// </summary>
    /// <typeparam name="T">The toInsert used to store the item managed by this manager</typeparam>
    public abstract class Manager<T>
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Manager&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="repository">The repository.</param>
        protected Manager(IRepository<T> repository)
        {
            this.Repository = repository;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets the repository.
        /// </summary>
        /// <value>The repository.</value>
        protected IRepository<T> Repository
        {
            get;
            private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns><c>True</c> if the data have passed the validation;
        /// otherwise <c>False</c></returns>
        public abstract bool ValidateData();

        /// <summary>
        /// Process to execute when a delete is raised.
        /// </summary>
        /// <param name="item">The item to delete.</param>
        protected virtual void Delete(T item)
        {
            this.Repository.Delete(item);
            AppContext.NotifyDataChanged();
        }

        /// <summary>
        /// Process to execute when a insertion is raised.
        /// </summary>
        /// <param name="item">The item to insert.</param>
        protected virtual void Insert(T item)
        {
            this.Repository.Create(item);
            AppContext.NotifyDataChanged();
        }

        /// <summary>
        /// Process to execute when an update is raised.
        /// </summary>
        /// <param name="item">The item to update.</param>
        protected virtual void Update(T item)
        {
            this.Repository.Update(item);
            AppContext.NotifyDataChanged();
        }

        #endregion Methods
    }
}