﻿namespace Phoenix.Model.Dal.Repositories
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    using NHibernate;
    using NHibernate.Linq;

    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;

    /// <summary>
    /// Provides CRUD features on patient.
    /// </summary>
    public class PatientRepository : Repository<Patient>, IPatientRepository
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PatientRepository"/> class.
        /// </summary>
        public PatientRepository()
            : base("Patient", "LastName")
        {
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Clears the specified patient from repository.
        /// That's, it'll remove all references of the specified
        /// patient from the repository
        /// </summary>
        /// <param name="patient">The patient to clear from repository.</param>
        public void ClearFromRepository(Patient patient)
        {
            patient.ClearDoctors();
            patient.ClearFamily();
        }

        /// <summary>
        /// Creates the specified item in the database.
        /// </summary>
        /// <param name="item">The item.</param>
        public override void Create(Patient item)
        {
            foreach (Doctor doctor in item.Doctors)
            {
                this.Session.SaveOrUpdate(doctor);
            }
            base.Create(item);
        }

        /// <summary>
        /// Gets the patient with the specified ID.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns></returns>
        public Patient GetById(long id)
        {
            return this.Session.Load<Patient>(id);
        }

        /// <summary>
        /// Gets the Debug patient.
        /// </summary>
        /// <returns></returns>
        public Patient GetDebugPatient()
        {
            var debug = (from Patient patient in this.Session.Linq<Patient>()
                         where patient.FirstName == "Jean-Baptiste" && patient.LastName == "WAUTIER"
                         select patient).FirstOrDefault();
            return debug;
        }

        /// <summary>
        /// Determines whether the specified item is referenced.
        /// A patient is never referenced.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns>
        /// 	<c>true</c> if the specified item is referenced; otherwise, <c>false</c>.
        /// </returns>
        public override bool IsReferenced(Patient item)
        {
            return false;
        }

        /// <summary>
        /// Searches the items in the database that respect the specified criterion.
        /// Every not empty properties are considered as criterion for the search.
        /// Every empty (null or with default values) are excluded of the search.
        /// The search is optimistic on text. For instance, that'll search on all
        /// the tuples that contains the text and not that has exactly the specified name.
        /// </summary>
        /// <param name="criterion">The criterion.</param>
        /// <returns>
        /// A list of the items that respect the criterion.
        /// </returns>
        //TODO Replace this method by a new one more staight forward
        [Obsolete("Replace this method by a new one more staight forward")]
        public IList<Patient> Search(Patient criterion)
        {
            return (criterion.LastName == "*")
                ? base.GetAllRows(OrderBy.ASC)
                : this.RegularSearch(criterion);
        }

        /// <summary>
        /// Updates the specified item against the database.
        /// </summary>
        /// <param name="item">The item.</param>
        public override void Update(Patient item)
        {
            foreach (var doctor in item.Doctors) this.Session.SaveOrUpdate(doctor);
            foreach (var card in item.MedicalCards) this.Session.SaveOrUpdate(card);
            base.Update(item);
        }

        private IList<Patient> RegularSearch(Patient criterion)
        {
            Assert.IsTrue((criterion != null), "The criterion of the search is null!");

            DateTime? dateOfBirth = new DateTime();
            DateTime? inscriptionDate = new DateTime();

            string hql = "FROM Patient p WHERE p.LastName LIKE :LastName AND p.FirstName LIKE :FirstName ";

            string firstName = (!string.IsNullOrEmpty(criterion.FirstName))
                                   ? criterion.FirstName
                                   : string.Empty;

            string lastName = (!string.IsNullOrEmpty(criterion.LastName))
                                  ? criterion.LastName
                                  : string.Empty;

            if (criterion.DateOfBirth > DateTime.MinValue)
            {
                dateOfBirth = criterion.DateOfBirth;
                hql += "AND p.DateOfBirth = :DateOfBirth ";
            }
            if (criterion.InscriptionDate > DateTime.MinValue)
            {
                inscriptionDate = criterion.InscriptionDate;
                hql += "AND p.InscriptionDate = :InscriptionDate";
            }
            hql += " ORDER BY p.LastName";

            IQuery query = this.Session.CreateQuery(hql);
            query.SetParameter("FirstName", firstName + "%");
            query.SetParameter("LastName", lastName.ToUpper() + "%");

            if (criterion.DateOfBirth > DateTime.MinValue)
                query.SetParameter("DateOfBirth", dateOfBirth);

            if (criterion.InscriptionDate != DateTime.MinValue)
                query.SetParameter("InscriptionDate", inscriptionDate);

            return query.List<Patient>();
        }

        #endregion Methods
    }
}