﻿namespace Phoenix.View.Input
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Presenter.Management;
    using Phoenix.Translation;
    using Phoenix.View.Helpers;
    using Phoenix.View.Helpers.Controls;

    /// <summary>
    /// Gui item to display a doctor.
    /// </summary>
    public partial class DoctorInput : TranslateableForm, IDoctorInputPresenter
    {
        #region Fields

        private const int width = 404;

        private Doctor itemToManage = new Doctor();
        private DoctorModel presenter;
        private List<DoctorType> specialisations = new List<DoctorType>();
        private Usage usage = Usage.Display;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DoctorInput"/> class.
        /// </summary>
        public DoctorInput()
        {
            this.presenter = new DoctorModel(this);
            this.Size = new Size();
            InitializeComponent();
            this.Usage = Usage.Display;
            this.Size = new Size(1, 1);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when a doctor isdeleted.
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when doctor is inserted.
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when the view is loading.
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when a doctor is updated.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public Doctor ItemToManage
        {
            get
            {
                this.itemToManage.Specialisation = tcb_Specialisation.SelectedItem as DoctorType;
                return this.itemToManage;
            }
            set
            {
                this.itemToManage = value;
                this.Refreshed.Raise(this);
                this.BindData();
            }
        }

        /// <summary>
        /// Gets a value indicating whether this manager is in read only mode.
        /// </summary>
        /// <value><c>true</c> if [read only]; otherwise, <c>false</c>.</value>
        public bool ReadOnly
        {
            get
            {
                return (this.Usage == Usage.Display || this.Usage == Usage.Delete);
            }
        }

        /// <summary>
        /// Gets or sets the specialisations.
        /// </summary>
        /// <value>The specialisations.</value>
        public List<DoctorType> Specialisations
        {
            get { return this.specialisations; }
            set
            {
                this.specialisations = value;
                this.tcb_Specialisation.Bind(value, "Title");
            }
        }

        /// <summary>
        /// Gets or sets the usage.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get { return this.usage; }
            set
            {
                this.usage = value;
                switch (this.usage)
                {
                    case Usage.Display:
                        this.btn_OK.Visible = false;
                        this.EnableControls(false);
                        this.btn_Cancel.Text = "Close".Translate();
                        break;
                    case Usage.Insert:
                        this.ItemToManage = new Doctor();
                        this.EnableControls(true);
                        this.btn_OK.Visible = true;
                        this.btn_OK.Text = "Insert".Translate();

                        this.btn_Cancel.Text = "Cancel".Translate();
                        break;
                    case Usage.Update:
                        this.btn_OK.Visible = true;
                        this.EnableControls(true);
                        this.btn_OK.Text = "Update".Translate();

                        this.btn_Cancel.Text = "Cancel".Translate();
                        break;
                    case Usage.Delete:
                        this.btn_OK.Visible = true;
                        this.EnableControls(false);
                        this.btn_OK.Text = "Delete".Translate();

                        this.btn_Cancel.Text = "Cancel".Translate();
                        break;
                    default:
                        Assert.FailsOnEnumeration(this.usage);
                        break;
                }
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears previous error on validation failed.
        /// </summary>
        public void ClearError()
        {
            foreach (Control control in this.flp_Panel.Controls)
            {
                var s = control.Size;
                control.Size = new Size(width, s.Height);
            }

            this.ep_Error.Clear();
        }

        /// <summary>
        /// Sets an error message to explain why data validation failed.
        /// </summary>
        /// <param name="failed">The fail.</param>
        /// <param name="message">The message.</param>
        public void SetError(DoctorFailOn failed, string message)
        {
            var control = new Control();
            switch (failed)
            {
                case DoctorFailOn.LastNameIsNum:
                    control = this.ttb_LastName;
                    break;
                case DoctorFailOn.LastNameIsEmpty:
                    control = this.ttb_LastName;
                    break;
                case DoctorFailOn.FirstNameIsNum:
                    control = this.ttb_FirstName;
                    break;
                case DoctorFailOn.FirstNameIsEmpty:
                    control = this.ttb_FirstName;
                    break;
                case DoctorFailOn.EmailIsNotValid:
                    control = this.ttb_Mail;
                    break;
                case DoctorFailOn.ProPhoneIsNotValid:
                    control = this.ttb_PostalCode;
                    break;
                case DoctorFailOn.ProMobileIsNotValid:
                    control = this.ttb_Mobile;
                    break;
                case DoctorFailOn.StreetIsNum:
                    control = this.ttb_Street;
                    break;
                case DoctorFailOn.CityIsNum:
                    control = this.ttb_City;
                    break;
                case DoctorFailOn.NoSpecialisation:
                    control = this.tcb_Specialisation;
                    break;
                case DoctorFailOn.Referenced:
                    PredifinedMsgBoxes.WarnOnReferencedItem();
                    break;
                default:
                    Assert.FailsOnEnumeration(failed);
                    break;
            }
            control.Width = width - 20;
            this.ep_Error.SetError(control, message);
        }

        /// <summary>
        /// Refreshes the data.
        /// </summary>
        private void BindData()
        {
            Assert.IsNotNull(this.itemToManage, "ItemToManage");

            /* I save the specialisation to setup the combobox the
             * first time to the specialisation of the doctor to bind.
             * If I don't do that, the first specialisation of the list
             * will always be selected.
             */
            var specialisation = this.itemToManage.Specialisation;

            foreach (Control control in this.flp_Panel.Controls)
            {
                control.Visible = false;
                control.DataBindings.Clear();
            }
            this.Size = this.MinimumSize;

            #region Doctor
            this.s_Doctor.Title = "Doctor".Translate();
            this.s_Doctor.Visible = true;

            this.ttb_FirstName.Value = this.ItemToManage.FirstName;
            this.ttb_FirstName.Visible = true;
            this.ttb_FirstName.DataBindings.Add("Text", this.itemToManage, "FirstName");

            this.ttb_LastName.Value = this.ItemToManage.LastName;
            this.ttb_LastName.Visible = true;
            this.ttb_LastName.DataBindings.Add("Text", this.itemToManage, "LastName");

            this.tcb_Specialisation.Visible = true;
            this.tcb_Specialisation.Bind(this.Specialisations, "Title");
            this.tcb_Specialisation.SelectedItem = specialisation;
            #endregion

            #region Address
            bool isAddressVisible = false;
            if (!string.IsNullOrEmpty(this.ItemToManage.Address.Street) || !this.ReadOnly)
            {
                this.ttb_Street.Value = this.ItemToManage.Address.Street;
                this.ttb_Street.Visible = true;
                isAddressVisible = true;
                this.ttb_Street.DataBindings.Add("Text", this.ItemToManage, "Address.Street");
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.Address.StreetNumber) || !this.ReadOnly)
            {
                this.ttb_StreetNumber.Value = this.ItemToManage.Address.StreetNumber;
                this.ttb_StreetNumber.Visible = true;
                isAddressVisible = true;
                this.ttb_StreetNumber.DataBindings.Add("Text", this.ItemToManage, "Address.StreetNumber");
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.Address.BoxNumber) || !this.ReadOnly)
            {
                this.ttb_BoxNr.Value = this.ItemToManage.Address.BoxNumber;
                this.ttb_BoxNr.Visible = true;
                isAddressVisible = true;
                this.ttb_BoxNr.DataBindings.Add("Text", this.ItemToManage, "Address.BoxNumber");
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.Address.City) || !this.ReadOnly)
            {
                this.ttb_City.Value = this.ItemToManage.Address.City;
                this.ttb_City.Visible = true;
                isAddressVisible = true;
                this.ttb_City.DataBindings.Add("Text", this.ItemToManage, "Address.City");
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.Address.PostalCode) || !this.ReadOnly)
            {
                this.ttb_PostalCode.Value = this.ItemToManage.Address.PostalCode;
                this.ttb_PostalCode.Visible = true;
                isAddressVisible = true;
                this.ttb_PostalCode.DataBindings.Add("Text", this.ItemToManage, "Address.PostalCode");
            }

            this.s_Address.Title = "Address".Translate();
            this.s_Address.Visible = isAddressVisible;
            #endregion

            #region Contact
            bool isContactVisible = false;
            if (!string.IsNullOrEmpty(this.ItemToManage.PhonePro) || !this.ReadOnly)
            {
                this.ttb_Phone.Value = this.ItemToManage.PhonePro;
                this.ttb_Phone.Visible = true;
                isContactVisible = true;
                this.ttb_Phone.DataBindings.Add("Text", this.ItemToManage, "PhonePro");
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.MobilePro) || !this.ReadOnly)
            {
                this.ttb_Mobile.Value = this.ItemToManage.MobilePro;
                this.ttb_Mobile.Visible = true;
                isContactVisible = true;
                this.ttb_Mobile.DataBindings.Add("Text", this.ItemToManage, "MobilePro");
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.Mail) || !this.ReadOnly)
            {
                this.ttb_Mail.Value = this.ItemToManage.Mail;
                this.ttb_Mail.Visible = true;
                isContactVisible = true;
                this.ttb_Mail.DataBindings.Add("Text", this.ItemToManage, "Mail");
            }

            this.s_Contact.Title = "Contact".Translate();
            this.s_Contact.Visible = isContactVisible;
            #endregion
        }

        private void btn_OK_Click(object sender, EventArgs e)
        {
            switch (this.Usage)
            {
                case Usage.Display:
                    this.Close();
                    break;
                case Usage.Insert:
                    if (this.presenter.ValidateData())
                    {
                        this.Inserted.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Update:
                    if (this.presenter.ValidateData())
                    {
                        this.Updated.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Delete:
                    this.Deleted.Raise(this);
                    this.DialogResult = DialogResult.OK;
                    this.Close();
                    break;
                default:
                    Assert.FailsOnEnumeration(this.Usage);
                    break;
            }
        }

        private void EnableControls(bool value)
        {
            foreach (var control in this.flp_Panel.Controls)
            {
                if (control is ITitledControl)
                    (control as ITitledControl).Enabled = value;
            }
        }

        #endregion Methods
    }
}