﻿namespace Phoenix.View.Input
{
    using System;
    using System.Drawing;
    using System.Linq;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;
    using Phoenix.View.Helpers;
    using Phoenix.View.Helpers.Controls;

    /// <summary>
    /// Gui tool to manage a practice
    /// </summary>
    public partial class PracticeInput : TranslateableForm, IPracticeInputPresenter
    {
        #region Fields

        private const int width = 404;

        private Practice practice = new Practice();
        private PracticeModel presenter;
        private Usage usage = Usage.Display;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PracticeInput"/> class.
        /// </summary>
        public PracticeInput()
        {
            this.presenter = new PracticeModel(this);
            this.Size = new Size();
            InitializeComponent();
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when user asked to delete an Practice
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when user asked to insert an Practice
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when user asked to update an Practice
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="InsuranceInput"/> is editable.
        /// </summary>
        /// <value><c>true</c> if editable; otherwise, <c>false</c>.</value>
        public bool Editable
        {
            set
            {
                foreach (var control in this.flp_Panel.Controls)
                {
                    if (control is ITitledControl) (control as ITitledControl).Enabled = value;
                }
            }
            get
            {
                return (from ITitledControl control in this.flp_Panel.Controls
                        select control
                    ).Count() > 0;
            }
        }

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public Practice ItemToManage
        {
            get
            {
                return this.practice;
            }
            set
            {
                this.practice = value;
                this.BindData();
            }
        }

        /// <summary>
        /// Gets or sets the usage of this GUI item.
        /// Should be Insert, Update or Delete.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get
            {
                return this.usage;
            }
            set
            {
                this.usage = value;
                switch (value)
                {
                    case Usage.Display:
                        this.btn_OK.Text = "OK".Translate();
                        this.Editable = false;
                        break;
                    case Usage.Insert:
                        this.ItemToManage = new Practice();
                        this.btn_OK.Text = "Insert".Translate();
                        this.DialogResult = DialogResult.OK;
                        this.Editable = true;
                        break;
                    case Usage.Update:
                        this.btn_OK.Text = "Update".Translate();
                        this.DialogResult = DialogResult.OK;
                        this.Editable = true;
                        break;
                    case Usage.Delete:
                        this.btn_OK.Text = "Delete".Translate();
                        this.DialogResult = DialogResult.OK;
                        this.Editable = false;
                        break;
                    default:
                        Assert.FailsOnEnumeration(value);
                        break;
                }
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears previous error on validation failed.
        /// </summary>
        public void ClearError()
        {
            foreach (Control control in this.flp_Panel.Controls)
            {
                var s = control.Size;
                control.Size = new Size(width, s.Height);
            }

            this.ep_Error.Clear();
        }

        /// <summary>
        /// Sets an error message on validation failed.
        /// </summary>
        /// <param name="fail">Indicates where the validation failed.</param>
        /// <param name="message">The message to display.</param>
        public void SetError(PracticeFailedOn fail, string message)
        {
            var control = new Control();
            switch (fail)
            {
                case PracticeFailedOn.EmptyTitle:
                    control = this.ttb_Name;
                    break;
                case PracticeFailedOn.Referenced:
                    PredifinedMsgBoxes.WarnOnReferencedItem();
                    break;
                default:
                    Assert.FailsOnEnumeration(fail);
                    break;
            }
            control.Width -= 20;
            this.ep_Error.SetError(control, message);
        }

        private void BindData()
        {
            foreach (Control control in this.flp_Panel.Controls)
            {
                control.Visible = false;
                control.DataBindings.Clear();
            }
            this.Size = this.MinimumSize;

            #region Practice
            this.s_Practice.Title = "Practice".Translate();
            this.s_Practice.Visible = true;

            this.ttb_Name.Value = this.ItemToManage.Title;
            this.ttb_Name.Visible = true;
            this.ttb_Name.DataBindings.Add("Text", this.ItemToManage, "Title");

            if (!string.IsNullOrEmpty(this.ItemToManage.Phone) && this.usage != Usage.Insert)
            {
                this.ttb_Phone.Value = this.ItemToManage.Phone;
                this.ttb_Phone.Visible = true;
                this.ttb_Phone.DataBindings.Add("Text", this.ItemToManage, "Phone");
            }
            #endregion

            #region Address
            bool isAddressVisible = false;
            if (!string.IsNullOrEmpty(this.ItemToManage.Address.Street) || this.usage == Usage.Insert)
            {
                this.ttb_Street.Value = this.ItemToManage.Address.Street;
                this.ttb_Street.Visible = true;
                this.ttb_Street.DataBindings.Add("Text", this.ItemToManage, "Address.Street");
                isAddressVisible = true;
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.Address.StreetNumber) || this.usage == Usage.Insert)
            {
                this.ttb_StreetNumber.Value = this.ItemToManage.Address.StreetNumber;
                this.ttb_StreetNumber.Visible = true;
                this.ttb_StreetNumber.DataBindings.Add("Text", this.ItemToManage, "Address.StreetNumber");
                isAddressVisible = true;
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.Address.BoxNumber) || this.usage == Usage.Insert)
            {
                this.ttb_BoxNr.Value = this.ItemToManage.Address.BoxNumber;
                this.ttb_BoxNr.Visible = true;
                this.ttb_BoxNr.DataBindings.Add("Text", this.ItemToManage, "Address.BoxNumber");
                isAddressVisible = true;
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.Address.City) || this.usage == Usage.Insert)
            {
                this.ttb_BoxNr.Value = this.ItemToManage.Address.City;
                this.ttb_City.Visible = true;
                this.ttb_City.DataBindings.Add("Text", this.ItemToManage, "Address.City");
            }

            if (!string.IsNullOrEmpty(this.ItemToManage.Address.PostalCode) || this.usage == Usage.Insert)
            {
                this.ttb_PostalCode.Value = this.ItemToManage.Address.PostalCode;
                this.ttb_PostalCode.Visible = true;
                this.ttb_PostalCode.DataBindings.Add("Text", this.ItemToManage, "Address.PostalCode");
            }

            this.s_Address.Title = "Address".Translate();
            this.s_Address.Visible = isAddressVisible;
            #endregion
        }

        private void btn_OK_Click(object sender, EventArgs e)
        {
            switch (this.Usage)
            {
                case Usage.Display:
                    this.Close();
                    break;
                case Usage.Insert:
                    if (this.presenter.ValidateData())
                    {
                        this.Inserted.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Update:
                    if (this.presenter.ValidateData())
                    {
                        this.Updated.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Delete:
                    if (this.presenter.ValidateData())
                    {
                        this.Deleted.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                default:
                    Assert.FailsOnEnumeration(this.Usage);
                    break;
            }
        }

        private void btn_OK_Click_1(object sender, EventArgs e)
        {
            switch (this.Usage)
            {
                case Usage.Display:
                    this.Close();
                    break;
                case Usage.Insert:
                    if (this.presenter.ValidateData())
                    {
                        this.Inserted.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Update:
                    if (this.presenter.ValidateData())
                    {
                        this.Updated.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Delete:
                    if (this.presenter.ValidateData())
                    {
                        this.Deleted.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                default:
                    Assert.FailsOnEnumeration(this.Usage);
                    break;
            }
        }

        #endregion Methods
    }
}