﻿namespace Phoenix.Presenter.Management
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Linq;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Manager;
    using Phoenix.Translation;
    using Phoenix.View;
    using Phoenix.View.Input;

    /// <summary>
    /// Provide management for the BMI history
    /// </summary>
    public partial class BmiManagerBox : TranslateableForm, IBmiManagerPresenter
    {
        #region Fields

        private const string LEGEND_1 = "Legend";
        private const string SERIE_BMI = "Bmi";
        private const string SERIE_HEIGHT = "Height";
        private const string SERIE_OVERWEIGHT = "Overweight";
        private const string SERIE_UNDERWEIGHT = "Underweight";
        private const string SERIE_WEIGHT = "Weight";
        private const string TITLE = "Title";

        private BmiModel presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BmiManagerBox"/> class.
        /// </summary>
        public BmiManagerBox()
        {
            this.Logger = LogManager.GetLogger(typeof(BmiManagerBox));
            this.presenter = new BmiModel(this);
            this.DateFormat = "dd/MM/yyyy";
            InitializeComponent();
            this.TranslateForm();

            this.chart_Bmi.Legends[LEGEND_1].InsideChartArea = "";
            this.chart_Height.Legends[LEGEND_1].InsideChartArea = "";
            this.chart_Weight.Legends[LEGEND_1].InsideChartArea = "";
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when the control is closing.
        /// </summary>
        public event EventHandler ControlClosing;

        /// <summary>
        /// Occurs when deleting a BMI entry.
        /// </summary>
        public event EventHandler<EventArgs<BMI>> Deleted;

        /// <summary>
        /// Occurs when the view is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the date format to display.
        /// </summary>
        /// <value>The date format.</value>
        public string DateFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the description of the manager.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return "Bmi history"; }
        }

        /// <summary>
        /// Gets the value of the icon tp show in the manager icon.
        /// </summary>
        /// <value>The icon.</value>
        public IconValue IconValue
        {
            get { return IconValue.Bmi; }
        }

        /// <summary>
        /// Gets the patient to manage.
        /// </summary>
        /// <value>The patient to manage.</value>
        public Patient Patient
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Called when the control has to be closed.
        /// </summary>
        public void OnControlClosing()
        {
            this.ControlClosing.Raise(this);
        }

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void OnRefreshed()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void RefreshData()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data.
        /// </summary>
        public void RefreshGui()
        {
            Assert.IsTrue(this.Patient != null, "Impossible to load the bmi history if the patient is not set!");
            if (chart_Bmi.Series.Count <= 0) return;

            this.lv_Bmi.Items.Clear();
            this.chart_Bmi.Series[SERIE_BMI].Points.Clear();
            this.chart_Bmi.Series[SERIE_OVERWEIGHT].Points.Clear();
            this.chart_Bmi.Series[SERIE_UNDERWEIGHT].Points.Clear();
            this.chart_Height.Series[SERIE_HEIGHT].Points.Clear();
            this.chart_Weight.Series[SERIE_WEIGHT].Points.Clear();

            var sortedBmi = this.Patient.BmiHistory.OrderBy(e => e.Date);
            List<double> YAxesBmi = new List<double>();
            List<double> YAxesObese = new List<double>();
            List<double> YAxesUnderweight = new List<double>();
            List<double> YAxesWeight = new List<double>();
            List<double> YAxesHeight = new List<double>();

            List<DateTime> XAxes = new List<DateTime>();

            foreach (var bmi in sortedBmi)
            {
                this.InsertListViewItem(bmi);

                YAxesBmi.Add(bmi.Index);
                YAxesObese.Add(25);
                YAxesUnderweight.Add(18.4);
                YAxesHeight.Add(bmi.Height);
                YAxesWeight.Add(bmi.Weight);

                XAxes.Add(bmi.Date);
            }
            this.chart_Bmi.Series[SERIE_BMI].Points.DataBindXY(XAxes, YAxesBmi);
            this.chart_Bmi.Series[SERIE_OVERWEIGHT].Points.DataBindXY(XAxes, YAxesObese);
            this.chart_Bmi.Series[SERIE_UNDERWEIGHT].Points.DataBindXY(XAxes, YAxesUnderweight);
            this.chart_Height.Series[SERIE_HEIGHT].Points.DataBindXY(XAxes, YAxesHeight);
            this.chart_Weight.Series[SERIE_WEIGHT].Points.DataBindXY(XAxes, YAxesWeight);
        }

        /// <summary>
        /// Closes this manager.
        /// </summary>
        public void ShutDown()
        {
            this.OnClosing(new CancelEventArgs());
            this.Close();
        }

        private void BmiManagerBox_Load(object sender, EventArgs e)
        {
            Assert.IsNotNull(this.Patient, "Patient", "The BmiManager should contain a patient!");

            this.lbl_Explanation.Text = lbl_Explanation.Text.Translate(this.Patient);
            this.Text = string.Format("Bmi: {0}", this.Patient);
            this.chart_Bmi.Titles[TITLE].Text = "BMI history of {0}.".Translate(this.Patient);
            this.chart_Weight.Titles[TITLE].Text = "Weight history of {0}.".Translate(this.Patient);
            this.chart_Height.Titles[TITLE].Text = "Height history of {0}.".Translate(this.Patient);

            RefreshGui();
        }

        private void cms_Menu_Opening(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (lv_Bmi.SelectedItems.Count == 0)
            {
                this.tsmi_DeleteBmi.Enabled = false;
                this.tsmi_EditBmi.Enabled = false;
            }
            else
            {
                this.tsmi_DeleteBmi.Enabled = true;
                this.tsmi_EditBmi.Enabled = true;
            }
        }

        private void DeleteSelectedItem()
        {
            if (lv_Bmi.SelectedItems.Count <= 0) return;

            BMI bmi = lv_Bmi.SelectedItems[0].Tag as BMI;
            Assert.IsTrue((bmi != null), "The selected node in the ListView should contain BMI whose not null!");

            BmiInput box = new BmiInput() { BmiToManage = bmi, Usage = Usage.Delete };
            DialogResult dr = box.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.Deleted.Raise(this, new EventArgs<BMI>(box.BmiToManage));
            this.RefreshGui();
        }

        private void InsertListViewItem(BMI bmi)
        {
            ListViewItem item = new ListViewItem(bmi.Date.ToString(this.DateFormat));
            ListViewItem.ListViewSubItem subHeight = new ListViewItem.ListViewSubItem(item, bmi.Height.ToString());
            ListViewItem.ListViewSubItem subWeight = new ListViewItem.ListViewSubItem(item, bmi.Weight.ToString());
            ListViewItem.ListViewSubItem subBmiIndice = new ListViewItem.ListViewSubItem(item, bmi.Index.ToString());
            ListViewItem.ListViewSubItem subMsg = new ListViewItem.ListViewSubItem(item, "");

            subMsg.Text = bmi.TextStatus.Translate();

            item.Tag = bmi;
            item.SubItems.Add(subHeight);
            item.SubItems.Add(subWeight);
            item.SubItems.Add(subBmiIndice);
            item.SubItems.Add(subMsg);
            lv_Bmi.Items.Add(item);
        }

        private void InsertNewItem()
        {
            BmiInput box = new BmiInput() { LastHeight = this.Patient.Height };
            DialogResult dr = box.ShowDialog();

            if (dr != DialogResult.OK) return;

            this.UpdatePatient(box.BmiToManage);
            this.Patient.Add(box.BmiToManage);
            this.Refreshed.Raise(this);
            this.RefreshGui();
        }

        private void lv_Bmi_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                this.DeleteSelectedItem();
            }
            else if (e.KeyCode == Keys.Insert)
            {
                this.InsertNewItem();
            }
        }

        private void tsmi_AddBmi_Click(object sender, EventArgs e)
        {
            this.InsertNewItem();
        }

        private void tsmi_DeleteBmi_Click(object sender, EventArgs e)
        {
            this.DeleteSelectedItem();
        }

        private void tsmi_EditBmi_Click(object sender, EventArgs e)
        {
            BMI bmi = lv_Bmi.SelectedItems[0].Tag as BMI;
            Assert.IsNotNull(bmi, "bmi");

            BmiInput box = new BmiInput() { BmiToManage = bmi, Usage = Usage.Update };
            DialogResult dr = box.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.UpdatePatient(box.BmiToManage);
            this.Refreshed.Raise(this);
            this.RefreshGui();
        }

        private void UpdatePatient(BMI bmi)
        {
            if (this.Patient.Height != bmi.Height)
            {
                this.Patient.Height = bmi.Height;
            }
        }

        #endregion Methods
    }
}