﻿namespace Phoenix.Contracts
{
    using System;
    using System.Diagnostics;

    /// <summary>
    /// Implements tools for programming by contracts which is
    /// an approach to designing computer software. It prescribes that software designers 
    /// should define formal, precise and verifiable interface specifications for software 
    /// components, which extend the ordinary definition of abstract data types with preconditions, 
    /// postconditions and invariants. These specifications are referred to as "contracts", in accordance with a 
    /// conceptual metaphor with the conditions and obligations of business contracts.
    /// </summary>
    public static class Assert
    {
        #region Fields

        private static Action<object, Exception> PostTreatment;

        #endregion Fields

        #region Methods

        /// <summary>
        /// Throws an <see cref="NotSupportedException"/> to indicate that the contract failed.
        /// </summary>
        /// <param name="message">The message to log in the thrown exception.</param>
        /// <param name="format">This will format the specified <c>message</c> with the specified <c>format</c> with a string.Format().</param>
        /// <returns>Does not return anything but can be used with a return statement</returns>
        /// <exception cref="NotSupportedException">This method throw always an exception with the specified text.</exception>        
        [DebuggerStepThrough]
        public static object Fails(object message, params object[] format)
        {
            message = message ?? "Contract failed";

            NotSupportedException ex = new NotSupportedException(string.Format(message.ToString(), format));
            HandlePostTreatment(message, ex);
            throw ex;
        }

        /// <summary>
        /// Throws an <see cref="NotSupportedException"/> to indicate that the contract failed on a not supported enumeration value.
        /// </summary>
        /// <param name="enumType">Type of the enum.</param>
        /// <exception cref="NotSupportedException">This method throw always an exception with the specified text.</exception>
        [DebuggerStepThrough]
        public static object FailsOnEnumeration(object enumType)
        {
            NotSupportedException ex = new NotSupportedException(
                string.Format("The enumeration type '{0}' is not supported!", enumType));
            HandlePostTreatment("Contract: enumeration fail", ex);
            throw ex;
        }

        /// <summary>
        /// Ensure that the specified argument is not null otherwise throws
        /// an <see cref="ArgumentNullException"/>.
        /// </summary>
        /// <param name="argument">The argument to check.</param>
        /// <param name="argumentName">Name of the argument to check.</param>
        /// <exception cref="ArgumentNullException">If the specfied argument is null</exception>
        [DebuggerStepThrough]
        public static void IsNotNull(object argument, string argumentName)
        {
            if (argument == null)
            {
                ArgumentNullException ex = new ArgumentNullException(argumentName
                    , string.Format("The contract failed: 'The specified argument '{0}' shouldn't be null!'", argumentName));
                HandlePostTreatment("Null argument failure", ex);
                throw ex;
            }
        }

        /// <summary>
        /// Ensure that the specified argument is not null otherwise throws
        /// an <see cref="ArgumentNullException"/>.
        /// </summary>
        /// <param name="argument">The argument to check.</param>
        /// <param name="argumentName">Name of the argument to check.</param>
        /// <param name="message">The message which explains the failure.</param>
        /// <exception cref="ArgumentNullException">If the specfied argument is null</exception>
        [DebuggerStepThrough]
        public static void IsNotNull(object argument, string argumentName, string message)
        {
            if (argument == null)
            {

                ArgumentNullException ex = new ArgumentNullException(argumentName
                    , string.Format("The contract failed: 'The specified argument '{0}' shouldn't be null: {1}'", argumentName, message));
                HandlePostTreatment("Null argument failure", ex);
                throw ex;
            }
        }

        /// <summary>
        /// Ensure that the workflow is correct by testing an assertion.
        /// If the assertion failed, throws an <see cref="NotSupportedException"/>
        /// </summary>
        /// <param name="assertion">if set to <c>false</c> throw an <see cref="NotSupportedException"/>.</param>
        /// <param name="message">The message.</param>
        /// <exception cref="NotSupportedException">If the assertion is false</exception>
        [DebuggerStepThrough]
        public static void IsTrue(bool assertion, object message)
        {
            if (!assertion)
            {
                string msg = string.Format("Contract assertion failed: '{0}'", message);
                NotSupportedException ex = new NotSupportedException(msg);
                HandlePostTreatment(message, ex);
                throw ex;
            }
        }

        /// <summary>
        /// Asserts the specified assertion.
        /// </summary>
        /// <param name="assertion">The assertion.</param>
        /// <param name="message">The message.</param>
        /// <exception cref="NotSupportedException">If the assertion is false</exception>
        [DebuggerStepThrough]
        public static void IsTrue(Func<bool> assertion, object message)
        {
            IsTrue(assertion(), message);
        }

        /// <summary>
        /// Ensure that specified type is the expected one.
        /// If the assertion failed, throws an <see cref="NotSupportedException"/>
        /// </summary>
        /// <param name="current">The current type.</param>
        /// <param name="expected">The expected type.</param>
        /// <param name="message">The message to display in the exception is assertion failed.</param>
        /// <exception cref="NotSupportedException">If the assertion is false</exception>
        [DebuggerStepThrough]
        public static void IsType(object current, Type expected, object message)
        {
            var type = current.GetType();

            if (type == expected) return;
            else if (!type.IsSubclassOf(expected))
            {
                string msg = string.Format("Contract type assertion failed: {0}"
                    , message);
                NotSupportedException ex = new NotSupportedException(msg);
                HandlePostTreatment(message, ex);
                throw ex;
            }
        }

        /// <summary>
        /// Ensure that specified type is the expected one.
        /// If the assertion failed, throws an <see cref="NotSupportedException"/>
        /// </summary>
        /// <param name="current">The current type.</param>
        /// <param name="expected">The expected type.</param>        
        /// <exception cref="NotSupportedException">If the assertion is false</exception>
        [DebuggerStepThrough]
        public static void IsType(object current, Type expected)
        {
            if (current == null) throw new NotSupportedException("Impossible to check the type of a NULL object.");

            string msg = string.Format("the expected type shoud be '{0}' but it is '{1}'"
                , expected, current.GetType());
            IsType(current, expected, msg);
        }

        public static void OnPostTreatment(Action<object, Exception> action)
        {
            PostTreatment = action;
        }

        private static void HandlePostTreatment(object message, Exception ex)
        {
            if (PostTreatment != null) PostTreatment(message, ex);
        }

        #endregion Methods
    }
}