﻿namespace Phoenix.Helpers.Misc
{
    using System;
    using System.Drawing;
    using System.IO;

    /// <summary>
    /// Provides tools to manage images.
    /// </summary>
    public static class ImageExtension
    {
        #region Methods

        /// <summary>
        /// Gets the thumbnail of the specified image with the specified width and height
        /// as maximum size.
        /// </summary>
        /// <param name="image">The image.</param>        
        /// <param name="maxHeight">Maximum height in pixels of the thumbnail.</param>
        /// <param name="maxWidth">Maximum width in pixels of the thumbnail.</param>
        /// <returns>
        /// An image that has the maximum specified height and with.
        /// The ratio of the specified image is respected.
        /// </returns>
        /// <exception cref="OutOfMemoryException">
        /// The file does not have a valid image format.  -or- GDI+ does not support
        /// the pixel format of the file.
        /// </exception>
        /// <exception cref="System.IO.FileNotFoundException">
        /// The specified file does not exist.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// filename is a System.Uri.
        /// </exception>
        public static Image GetThumbnailImage(this Image image, int maxHeight, int maxWidth)
        {
            if (image == null) return new Bitmap(1, 1);
            Image bmp = image;
            Size size = GetThumbnailSize(bmp, maxHeight, maxWidth);
            return bmp.GetThumbnailImage(size.Width, size.Height, () => false, IntPtr.Zero);
        }

        /// <summary>
        /// Gets the image into a bytes array.
        /// </summary>
        /// <param name="image">The image to process.</param>
        /// <returns>A bytes array representing the image.</returns>
        /// <exception cref="System.ArgumentNullException">
        /// stream or format is null.
        /// </exception>
        /// <exception cref="System.Runtime.InteropServices.ExternalException">
        /// The image was saved with the wrong image format
        /// </exception>
        public static byte[] ToBytes(this Image image)
        {
            if (image == null) return null;
            MemoryStream ms = new MemoryStream();
            image.Save(ms, System.Drawing.Imaging.ImageFormat.Gif);
            return ms.ToArray();
        }

        private static float DeterminePercentageForResize(int height, int width)
        {
            int highestValue;
            if (height > width)
                highestValue = height;
            else
                highestValue = width;

            float percent = 100 / (float)highestValue;

            if (percent > 1 && percent != 0)
                throw new Exception("Percent cannot be greater than 1 or equal to zero");
            else
                return percent;
        }

        private static Size GetThumbnailSize(Image original, int maxHeight, int maxWidth)
        {
            // Maximum size of any dimension.
            int maxPixels = (maxHeight > maxWidth)
                ? maxWidth
                : maxHeight;

            // Width and height.
            int originalWidth = original.Width;
            int originalHeight = original.Height;

            // Compute best factor to scale entire image based on larger dimension.
            double factor;
            if (originalWidth > originalHeight)
            {
                factor = (double)maxPixels / originalWidth;
            }
            else
            {
                factor = (double)maxPixels / originalHeight;
            }

            // Return thumbnail size.
            return new Size((int)(originalWidth * factor), (int)(originalHeight * factor));
        }

        #endregion Methods
    }
}