﻿namespace Phoenix.Translation
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.IO;
    using System.Reflection;
    using System.Windows.Forms;

    using log4net;

    /// <summary>
    /// Provides translation features
    /// </summary>
    public static class Translator
    {
        #region Fields

        private static Dictionary<string, string> dictionary;
        private static bool enabled = true;
        private static Language? language = null;
        private static IRepository repository;
        private static Stopwatch stopWatch = new Stopwatch();

        #endregion Fields

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="Translator"/> is enabled.
        /// </summary>
        /// <value><c>true</c> if enabled and try to translate; otherwise, <c>false</c> and works as string.Format().</value>
        public static bool Enabled
        {
            get { return enabled; }
            set { enabled = value; }
        }

        /// <summary>
        /// Gets the time in miliseconds of the last translation caching.
        /// </summary>
        /// <value>The time to load.</value>
        public static long TimeToLoad
        {
            get;
            private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Gets the DDL (Data Definition Language) code to create a translation database.        
        /// </summary>                
        /// <returns>The SQL to create a new translation database.</returns>
        public static string GetDDL()
        {
            string assemblyName = typeof(Translator).Assembly.GetName().Name;
            string fileName = "Resources.Translation.db.sql";
            string retString;

            Assembly assembly = Assembly.Load(assemblyName);
            using (Stream str = assembly.GetManifestResourceStream(assemblyName + "." + fileName))
            {

                if (str == null)
                    throw new IOException("Could not locate embedded resource '" + fileName + "' in assembly '" + assemblyName + "'");
                retString = new StreamReader(str).ReadToEnd();
            }
            return retString;
        }

        /// <summary>
        /// Loads the transaltor with specified translation repository.
        /// Will access the database only if the translator is not configured.
        /// <param name="translationRepository">The translation repository that 
        /// fetch the translation.</param>
        public static void LazyLoad(IRepository translationRepository)
        {
            LazyLoad(translationRepository, null);
        }

        /// <summary>
        /// Loads the transaltor with specified translation repository.
        /// Will access the database only if the translator is not configured.
        /// </summary>
        /// <param name="translationRepository">The translation repository that 
        /// fetch the translation.</param>
        /// <param name="languageToSet">The languageToSet for the translation.</param>
        public static void LazyLoad(IRepository translationRepository, Language? language)
        {
            if (repository == null)
                Load(translationRepository, language);
        }

        /// <summary>
        /// Loads the translator with specified translation repository.
        /// Will access anyways the database.
        /// </summary>
        /// <param name="translationRepository">The translation repository.</param>
        public static void Load(IRepository translationRepository)
        {
            Load(translationRepository, null);
        }

        /// <summary>
        /// Loads the transaltor with specified translation repository.
        /// Will access anyways the database.
        /// </summary>
        /// <param name="translationRepository">The translation repository that 
        /// fetch the translation.</param>
        /// <param name="languageToSet">The languageToSet for the translation.</param>
        public static void Load(IRepository translationRepository, Language? languageToSet)
        {
            if (translationRepository == null)
                throw new ArgumentException("The translation repository shouldn't be null!", "translationRepository");

            stopWatch.Reset();
            stopWatch.Start();
            repository = translationRepository;

            if (languageToSet != null)
            {
                language = languageToSet;
                translationRepository.SetLanguage(language.Value);
            }
            dictionary = repository.GetTranslations();
            stopWatch.Stop();
            TimeToLoad = stopWatch.ElapsedMilliseconds;
        }

        /// <summary>
        /// Sets the languageToSet of the translator.
        /// </summary>
        /// <param name="languageToSet">The languageToSet to set.</param>
        /// <exception cref="NullReferenceException">If the repository was never set before the call
        /// of this method.</exception>
        public static void SetLanguage(Language languageToSet)
        {
            if (repository == null)
            {
                string message = "The translator does not have any configured repository"
                    + ", impossible to set the language to the translator!";
                throw new ArgumentNullException("languageToSet", message);
            }
            else
            {
                language = languageToSet;
                Load(repository, languageToSet);
            }
        }

        /// <summary>
        /// Translates the specified format with the specified arguments into the languageToSet
        /// set from the Translator.
        /// </summary>
        /// <param name="format">The format.</param>
        /// <param name="args">The args.</param>
        /// <returns>The translated formated text.</returns>
        [DebuggerStepThrough]
        public static string Translate(this string format, params object[] args)
        {
            if (Enabled)
            {
                if (dictionary == null || repository == null)
                {
                    return CheckAndFormat(format, args);
                }
                else
                {
                    string key = format.ToLower();
                    if (string.IsNullOrEmpty(key)) return string.Empty;

                    if (dictionary == null || dictionary.Count == 0)
                    {
                        dictionary = repository.GetTranslations();
                    }
                    if (!dictionary.ContainsKey(key))
                    {
                        if (!repository.ExistInRepository(key))
                            InsertNewTranslation(format);
                        return CheckAndFormat(format, args);
                    }
                    else
                    {
                        format = dictionary[key];
                        return CheckAndFormat(format, args);
                    }
                }
            }
            else
            {
                return string.Format(format, args);
            }
        }

        /// <summary>
        /// Translates recusively the specified control.
        /// </summary>
        /// <param name="control">The control.</param>
        public static void Translate(object toTranslate)
        {
            #region Control to NOT translate
            if (toTranslate is DateTimePicker
                || toTranslate is TextBox
                || toTranslate is ComboBox)
            {
                return;
            }
            #endregion
            #region MenuStrip
            if (toTranslate is MenuStrip)
            {
                MenuStrip mStrip = toTranslate as MenuStrip;
                mStrip.Text.Translate();
                foreach (var item in mStrip.Items)
                {
                    Translator.Translate(item);
                }
            }
            #endregion
            #region Control
            else if (toTranslate is Control)
            {
                Control control = toTranslate as Control;
                if (!(control is TextBox || control is RichTextBox))
                    control.Text = control.Text.Translate();

                foreach (Control item in control.Controls)
                {
                    Translator.Translate(item);
                }
            }
            #endregion
            #region ToolStripDropDownItem
            else if (toTranslate is ToolStripDropDownItem)
            {
                ToolStripDropDownItem tsddi = toTranslate as ToolStripDropDownItem;
                tsddi.Text = tsddi.Text.Translate();
                foreach (var item in tsddi.DropDownItems)
                {
                    var temp = item as ToolStripDropDownItem;
                    if (temp != null)
                        temp.Text = temp.Text.Translate();

                    Translator.Translate(item);
                }
            }
            #endregion
            #region ToolStripItem
            else if (toTranslate is ToolStripItem)
            {
                ToolStripItem tssi = toTranslate as ToolStripItem;
                tssi.Text = tssi.Text.Translate();

            }
            #endregion
            #region Not translated
            else
            {
                FieldInfo fieldInfo = toTranslate.GetType().GetField("Name");
                string name = (fieldInfo == null) ? "N.A." : fieldInfo.Name;

                string msg = string.Format("Impossible to translate this component '{0}' of type '{1}'"
                    , toTranslate.GetType().Name, name);

                ILog logger = LogManager.GetLogger(typeof(Translator));
                logger.Debug(msg);
            }
            #endregion
        }

        private static string CheckAndFormat(string format, object[] args)
        {
            string msg = string.Empty;
            try
            {
                if (args.Length == 0)
                    return format;
                else
                    return string.Format(format, args);
            }
            catch (ArgumentNullException ex)
            {
                msg = string.Format("The specified argument '{0}' is null and the translation is impossible!"
                    , ex.ParamName);
                throw new ArgumentNullException(msg, ex);
            }
            catch (FormatException ex)
            {
                msg = string.Format("Impossible to format the string '{0}' for translation! Check the string to format."
                    , format);
                throw new FormatException(msg, ex);
            }
        }

        private static void InsertNewTranslation(string key)
        {
            repository.InsertNewPhrase(key);
        }

        #endregion Methods
    }
}